#ifndef lint
static char *rcsid = "$Header: /a/izar/vol/fileservers/izar/izar4/src/local/X500/xd/work.motif/src/RCS/browse.c,v 1.1 94/02/01 16:37:13 ccsrdsm Exp Locker: ccsrdsm $";
#endif

/* $Header: /a/izar/vol/fileservers/izar/izar4/src/local/X500/xd/work.motif/src/RCS/browse.c,v 1.1 94/02/01 16:37:13 ccsrdsm Exp Locker: ccsrdsm $ */

/***************************************************************************

  browse.c - List directory entries.

***************************************************************************/

#include "defs.h"

#include "query.h"
#include "quipu/common.h"

#include "windows.h"
#include "objects.h"
#include "callbacks.h"
#include "configure.h"
#include "actions.h"
#include "y.tab.h"
#include "layout.h"
#include "xdutil.h"
#include "showlist.h"
#include "requests.h"
#include "status.h"
#include "showentry.h"
#include "errview.h"

/**************************************************
 *
 * Externals..
 * 
 */

extern Display *dpy;

/**************************************************
 *
 * Local functions.
 * 
 */

static void gotBrowseResult();

/*****************************************************************************
 * doBrowseAction()
 *
 * Perform browse action for given window object.
 *
 ****************************************************************************/

void doBrowseAction(prim_widget,
		    object_id,
		    prim_id,
		    action_id,
		    actlist)
     Widget prim_widget;
     int object_id;
     int prim_id;
     int action_id;
     XLUactionList actlist;
{
  XLUwindowObject object;
  QyErrorCode err;
  int req_id;
  stringCell select;

  object = XLUgetWindowObject(object_id);

  if (object->current_entry == NULLEntryList)
    {
      XLUstatusMessage(object_id, "No entry given to browse under!");
      return;
    }

  XLUstatusMessage(object_id, "Browsing, please wait.");

  select = GetFullSelection();
  err = QyBrowse(object->current_entry->string_dn, select, &req_id);

  switch (err)
    {
    case QERR_bad_name:
      
      XLUstatusMessage(object_id, "Invalid entry name!");
      return;

    case QERR_internal_limit_reached:

      XLUstatusMessage(object_id, "Internal limit reached.");
      return;
    }

  /**************************************************
   *
   * Record request invocation.
   * 
   */

    XLUrequestInvoked(req_id,
		    BROWSE_ENTRY,
		    gotBrowseResult,
		    (DSReportCallback) NULL,
		    prim_widget,
		    object_id,
		    prim_id,
		    action_id,
		    actlist);
} /* doBrowseAction */

/*****************************************************************************
 * gotBrowseResult()
 *
 * 
 *
 ****************************************************************************/

/* ARGSUSED */
static void gotBrowseResult(req_id, object_id, action_id, actlist)
     int req_id;
     int object_id;
     int action_id;
     XLUactionList actlist;
{
  browseResults result;
  char errmsg[LINESIZE], rdnbuf[LINESIZE];
  int child_id, tmpl_id;
  String end;

  XLUstatusMessage(object_id, "Browse result received.");
  result = get_browse_results(req_id);
  
  QyPrintFriendlyDN(result->baseobject, rdnbuf);
  for (end = rdnbuf; !isnull(*end) && *end != ','; end++)
    ;

  *end = '\0';

  XBell(dpy, 40);

  if (result->entries == NULLEntryList)
    {
      if (result->errors == NULLError)
	{
	  (void) sprintf(errmsg, "Nothing found in entry `%s'!", rdnbuf);
	}
      else
	{
	  (void) sprintf(errmsg,
			 "Couldn't browse under `%s..' because;\n%s",
			 rdnbuf,
			 QyPrintErrcode(result->errors->error_type));
	}
    }
  else
    {
      int hitnum;
      entryList pentry;

      for (pentry = result->entries, hitnum = 0;
	   pentry != NULLEntryList;
	   pentry = pentry->next, hitnum++)
	;

      if (hitnum == 1)
	{
	  (void) XLUgetOutputTemplateID(actlist->outputs,
					XLUoneHit,
					&tmpl_id);

	  (void) XLUfindOutputObject(object_id,
				     tmpl_id,
				     actlist->mode,
				     &child_id);
	  
	  (void) sprintf(errmsg, "Found 1 entry under `%s'.", rdnbuf);
	  
	  XLUsetWindowEntry(child_id, result->entries);
	  displayCurrentEntry(NULLWidget, child_id, 0, 0, NULLActionList);
	  
	  doPrimitiveSensitivity(child_id);
	}
      else if (hitnum > 1)
	{
	  (void) XLUgetOutputTemplateID(actlist->outputs,
					XLUmanyHits,
					&tmpl_id);

	  (void) XLUfindOutputObject(object_id,
				     tmpl_id,
				     actlist->mode,
				     &child_id);
	  
	  (void) sprintf(errmsg,
			 "Found %d entries under `%s'.",
			 hitnum,
			 rdnbuf);
	  
	  XLUsetWindowList(child_id, result->entries);
	  displayCurrentList(child_id);
	  
	  doPrimitiveSensitivity(child_id);
	}
    }

  XLUsetWindowErrors(object_id, result->errors);
  result->errors = NULLError;

  /**************************************************
   *
   * Processed results. Display output.
   * 
   */

  updateErrorViewer(object_id);
  XLUstatusMessage(object_id, errmsg);
	    
  XLUrequestComplete(req_id);
  browse_results_free(&result);  
} /* gotBrowseResult */







