%{
#include "defs.h"
#include "layout.h"
#include "configure.h"
#include "actions.h"
#include "status.h"
#include "pipeout.h"
#include "dn_list.h"
#include "query.h"
#include "sequence.h"
#include "ufsearch.h"
#include "read.h"
#include "attrt_list.h"

int yylex();
void exit();

extern FILE *configuration_file;
extern int config_line_num;
extern char config_file_path[];
extern XLUreadAttribute attributes_to_display;
extern String default_target;
extern char bitmap_directory[], help_directory[], XLUresfile[];

static char lexeme[255];

%}
%start configure

%union
{
  char strval[255];
  int symbol;
  XLUlayoutObject layout;
  orientation orient;
  XLUwindowMode mode;
  XLUobjectProperty resource;
  XLUactionList actlist;
  XLUactionType atype;
  stringCell string_list;
  entryList entry_list;
  int number;
  attrTypeList atlist;
  XLUlistValue listval_list;
  QyFilter filter;
}

%token HORIZONTAL VERTICAL START ACTION BITMAP HELPFILE
%token WIDTH HEIGHT EXPAND INSENSITIVE LABEL NAME
%token HELP_DIR BITMAP_DIR APPDEF_FILE
%token A_RAISEOBJ A_KEEPOBJ A_ABORT A_CLOSEOBJ
%token A_QUITAPP A_ACTIVATE A_PIPEOUT A_SELECT
%token MODIFY_TYPE ATTR_TEMPLATE ATTR_GROUP
%token UFN_PATH UFS_CLASS UFS_TARGET UFS_DEFAULTS DEFAULT_TARGET
%token LAYOUT DNDB NEW SINGLE GLOBALMANAGED LOCALMANAGED CURRENT
%token READ_TYPE ATTR_NAME LIST_ATTR
%token <symbol> STATUS TITLE ENTRY_DISPLAY LOOKUP ENTRY_LIST HELP
%token <symbol> MOVETO BUTTON MODIFY ERROR_VIEW BROWSE SEARCH_LOG
%token <symbol> ADD
%token <strval> STRING

%type <layout> 		layout layout_list
%type <symbol> 		primitive_window activateable
%type <mode> 		output_mode
%type <orient> 		orient
%type <resource> option option_def option_list prim_options layout_options
%type <actlist> 	action_list action_def
%type <string_list> 	string_list
%type <entry_list>	entry_list oc_entry_list
%type <number> 	ufn_bound
%type <atlist> 	attrlist
%type <filter> 	andlist andlisttail filter
%type <listval_list>	listval_list listval

%%

/**************************************************
 *
 * Main rule.
 * 
 */

 configure:	var_list
      	   	;

/**************************************************
 *
 * The configure options
 * 
 */

 var_list:	var
  		| var_list var
  		;

 var:		LAYOUT ':' layout_def		{;}

		| START ':' start_list		{;}

		| ACTION ':' action_def		{;}

		| DNDB ':' dn_def		{;}

                | modify_option			{;}

		| read_option			{;}
	
                | attr_name_option		{;}

		| ufn_path_option		{;}
	
		| ufs_path_option		{;}
	
		| ufs_target_class_option	{;}
	
		| ufs_cls_option		{;}

		| ufs_default_option		{;}
	
		| LIST_ATTR ':' list_attr_option {;}
	
		| bitmaps_option		{;}
	
		| helps_option			{;}

		| appdef_option			{;}
	
		;

/**************************************************
 *
 * Define directories and files.
 * 
 */

 bitmaps_option:	BITMAP_DIR ':' STRING
			  {
			    (void) strcpy(bitmap_directory, $3);
			  }
			;

 helps_option:		HELP_DIR ':' STRING
			  {
			    (void) strcpy(help_directory, $3);
			  }
			;

 appdef_option:		APPDEF_FILE ':' STRING
			  {
			    (void) strcpy(XLUresfile, $3);
			  }
			;

/**************************************************
 *
 * Rule defining list of windows to appear
 * on startup.
 * 
 */

 start_list:	start
  		| start_list ',' start

 start:   	STRING
		  {
		 	add_to_startup_list(add_template_entry($1));
		  }
                ;

/**************************************************
 *
 * Layout definition rules for  an XLookup window
 * 
 */

 layout_def:	STRING ':' orient '{' layout_list '}'
		  {
		    	$5->orient = $3;
			put_layout(add_template_entry($1), $5);
		  }


 orient:	HORIZONTAL	{$$ = horizontal;}
		| VERTICAL 	{$$ = vertical;}
		;

 layout_list:	layout 
		  {
		    $$ = $1;
		  }
  		| layout_list layout
		  {
		    XLUlayoutObject layout;

		    for (layout = $1;
			 layout != NULLLayout && layout->next != NULLLayout;
			 layout = layout->next)
		      ;
		    
		    if (layout != NULLLayout)
		      {
			layout->next = $2;
			$$ = $1;
		      }
		    else
		      $$ = $2;
		  }  		  
  		;

 layout:	STRING layout_options
	 	  {
		    $$ = add_branch_to_layout(add_template_entry($1), $2);
		  }

		| primitive_window prim_options
		  {
		    $$ = add_prim_to_layout($1, $2);
		  }

		;

/**************************************************
 *
 * The flags and variables associated with
 * window objects.
 * 
 */

 layout_options: '(' option_list ')'  	{$$ = $2;}
		|			{$$ = NULLObjRes;}
		;


 prim_options:	'(' option_list ')'  	{$$ = $2;}
		| 			{$$ = NULLObjRes;}
		;

 option_list:	option			{$$ = $1;}
		| option_list option	{$2->next = $1; $$ = $2;}

 option:	'(' option_def ')'  	{$$ = $2;}
		;

 option_def: 	BITMAP STRING
		  {
		    $$ = put_resource(OT_bitmap, (XtPointer) $2);
		  }

		| HELPFILE STRING
		  {
		    $$ = put_resource(OT_helpfile, (XtPointer) $2);
		  }

		| HEIGHT STRING
		  {
		    $$ = put_resource(OT_height, (XtPointer) $2);
		  }

		| WIDTH STRING 
		  {
		    $$ = put_resource(OT_width, (XtPointer) $2);
		  }

		| EXPAND
		  {
		    $$ = put_resource(OT_expand, (XtPointer) NULL);
		  }
	
		| INSENSITIVE '(' string_list ')'
		  {
		    $$ = put_resource(OT_insensitive, (XtPointer) $3);
		  }

		| LABEL STRING
		  {
		    $$ = put_resource(OT_label, (XtPointer) $2);
		  }

		| NAME STRING
		  {
		    $$ = put_resource(OT_name, (XtPointer) $2);
		  }
		;

/**************************************************
 *
 * DN database
 * 
 */

 dn_def:	STRING ':' STRING
		  {
		  	add_dn_db($1, $3);
		  }
		;

/**************************************************
 *
 * Rules for actions definitions
 * 
 */

 action_def:    STRING ':' '(' action_list
		  {
			add_action($1, $4);
		  }
                ;

 action_list:	action_def ',' action_list 	{$1->next = $3; $$ = $1;}
		| action_def ')'   		{$$ = $1;}
		;			

 action_def:	A_ABORT '(' ')'
                  {
		    $$ = new_actlist(AT_AbortRequest,
				   NULLString,
				   CurrentWin,
				   (XtPointer) NULL,
				   NULLStrCell);
		  }
  
		| A_RAISEOBJ '(' string_list ',' output_mode ')'
		  {
		    $$ = new_actlist(AT_RaiseObject,
				   NULLString,
				   $5,
				   (XtPointer) NULL,
				   $3);

		    SeqFree(&$3);
		  }

		| A_KEEPOBJ '(' ')'
		  {
		    $$ = new_actlist(AT_KeepObject,
				   NULLString,
				   CurrentWin,
				   (XtPointer) NULL,
				   NULLStrCell);
		  }

		| A_CLOSEOBJ '(' ')'
		  {
		    $$ = new_actlist(AT_CloseObject,
				   NULLString,
				   CurrentWin,
				   (XtPointer) NULL,
				   NULLStrCell);
		  }

		| A_PIPEOUT '(' STRING ',' string_list ')'
  		  {
		    XLUprocessSpec spec;

		    spec = XtNew(struct XLUprocessSpec);
		    spec->execname = XtNewString($3);
		    spec->attrlist = $5;

		    $$ = new_actlist(AT_Pipeout,
				     NULLString,
				     CurrentWin,
				     (XtPointer) spec,
				     NULLStrCell);
		  }

		| A_SELECT '(' ')'
		  {
		    $$ = new_actlist(AT_Select,
				     NULLString,
				     CurrentWin,
				     (XtPointer) NULL,
				     NULLStrCell);
		  }

		| A_QUITAPP '(' ')'
		  {
		    $$ = new_actlist(AT_QuitApp,
				     NULLString,
				     CurrentWin,
				     (XtPointer) NULL,
				     NULLStrCell);
		  }

		| A_ACTIVATE  '(' activateable ',' STRING ',' output_mode ',' string_list ')'
		  {
		    $$ = new_actlist(AT_Activate, $5, $7, (XtPointer) $3, $9);
		    SeqFree(&$9);
		  }
		;


 activateable: 	LOOKUP
  		 {
		   $$ = $1;
		 }

		| MOVETO
  		 {
		   $$ = $1;
		 }

		| MODIFY
  		 {
		   $$ = $1;
		 }

		| ENTRY_DISPLAY 
  		 {
		   $$ = $1;
		 }

		| BROWSE 
  		 {
		   $$ = $1;
		 }

		| ADD 
  		 {
		   $$ = $1;
		 }
		;

			     
 output_mode:   NEW		{$$ = NewAlways;}

                | SINGLE	{$$ = NewWhenNone;}

		| GLOBALMANAGED	{$$ = NewWhenKeptGlobal;}

		| LOCALMANAGED	{$$ = NewWhenKeptLocal;}

		| CURRENT	{$$ = CurrentWin;}
		;

/**************************************************
 *
 * Defintions for primitive objects
 * 
 */

 primitive_window:	BUTTON		{$$ = $1;}
		   
			| ENTRY_LIST	{$$ = $1;}

			| ERROR_VIEW 	{$$ = $1;}

			| SEARCH_LOG 	{$$ = $1;}

		   	| HELP 		{$$ = $1;}

			| LOOKUP	{$$ = $1;}

		   	| MOVETO	{$$ = $1;}

		   	| MODIFY	{$$ = $1;}

		   	| ENTRY_DISPLAY	{$$ = $1;}

		   	| STATUS	{$$ = $1;}

  		   	| TITLE		{$$ = $1;}
		   ;

/**************************************************
 *
 * Directory configuration rules
 * 
 */

/**************************************************
 *
 * UFN paths
 *
 */

 ufn_path_option:	UFN_PATH ':' ufn_bound ',' ufn_bound ':' entry_list
			  {
			    QyUFNAddPath($3, $5, $7);
			  }

			| UFN_PATH ':' ufn_bound ':' entry_list
			  {
			    QyUFNAddPath($3, _QyUFNUnbounded, $5);
			  }
			;

 ufn_bound:	STRING 
		  {
		    if (!isdigit(*($1)))
		      $$ = _QyUFNUnbounded;
		    else
		      {
			int c;

			long strtol();

			c = (int) strtol((char *) $1, (char **) NULL, 10);
			$$ = c;
		      }
		  }

		| '+'
 	  	  {
		    $$ = _QyUFNPlus;
		  }

		;

 entry_list:	STRING
		  {
		    String dn;
		    $$ = NULLEntryList;
	
		    dn = get_dn_db($1);
		    (void) dn_list_insert(dn, &($$), NULLAVList, NULLAttrT);
		  }

		| STRING ',' entry_list 
		  {
		    String dn;
		    $$ = $3;
	
		    dn = get_dn_db($1);
		    (void) dn_list_insert(dn, &($$), NULLAVList, NULLAttrT);
		  }

		;

 oc_entry_list: STRING STRING 
                  {
		    AttributeType oc_type;
		    String dn;

		    oc_type = str2AttrT(name2oc($2)->oc_ot.ot_stroid);
		    if (oc_type == NULLAttrT)
		      {
			char msg[LINESIZE];

			(void) sprintf(msg,
             "\"%s\" - Error at or before line %d: Invalid objectclass `%s'.",
				       config_file_path,
				       config_line_num,
				       $2);

			XLUpopupMessage(msg);
			exit(1);
		      }

		    $$ = NULLEntryList;
		    dn = get_dn_db($1);

		    (void) dn_list_insert(dn, &($$), NULLAVList, oc_type);
		  }

                | STRING STRING ',' oc_entry_list 
		  {
		    AttributeType oc_type;
		    String dn;
		    
		    oc_type = str2AttrT(name2oc($2)->oc_ot.ot_stroid);
		    if (oc_type == NULLAttrT)
		      {
			char msg[LINESIZE];

			(void) sprintf(msg,
  "\"%s\" - Error at or before line %d: Invalid objectclass `%s'.",
				       config_file_path,
				       config_line_num,
				       $2);

			XLUpopupMessage(msg);
			exit(1);
		      }

		    $$ = $4;
		    dn = get_dn_db($1);

		    (void) dn_list_insert(dn, &($$), NULLAVList, oc_type);
		  }

		; 

/***************************************************
 *
 * Form search paths
 *
 */

 ufs_default_option: DEFAULT_TARGET ':' STRING
		       {
			 default_target = QyCopyString($3);
		       }
   		     ;

 ufs_path_option: UFS_DEFAULTS ':' STRING ':' string_list ':' oc_entry_list
		    {
		      QyAddFormPath($3, $5, $7);
		    }
		  | UFS_DEFAULTS ':' STRING ':' string_list ':' 
		    {
		      QyAddFormPath($3, $5, NULLEntryList);
		    }
		  | UFS_DEFAULTS ':' STRING ':' ':' oc_entry_list
		    {
		      QyAddFormPath($3, NULLStrCell, $6);
		    }
		  | UFS_DEFAULTS ':' STRING ':' ':'
		    {
		      QyAddFormPath($3, NULLStrCell, NULLEntryList);
		    }
 		  ;
	
 ufs_target_class_option: UFS_TARGET ':' STRING ':' string_list
                           {
			     if (QyAddTargetClass($3, $5) == FALSE)
			       {
				 char msg[LINESIZE];

				 (void) sprintf(msg,
"\"%s\" - Error at line %d: Maximum number of allowed searchTargets exceeded.",
						config_file_path,
						config_line_num);

				 XLUpopupMessage(msg);
				 exit (1);
			       }
			   }

			  | UFS_TARGET ':' STRING ':'
			   {
			     if (QyAddTargetClass($3, NULLStrCell) == FALSE)
			       {
				 char msg[LINESIZE];

				 (void) sprintf(msg,
"\"%s\" - Error at line %d: Maximum number of allowed searchTargets exceeded.",
						config_file_path,
						config_line_num);

				 XLUpopupMessage(msg);
				 exit (1);
			       }
			   }
			;

	
 ufs_cls_option: UFS_CLASS ':' STRING ':' STRING ':' filter ':' string_list
                   {
		     add_ufs_search_class($3, $5, $7, $9);
		     SeqFree(&$9);
		   }
		;

 filter:	andlist
		  {
		    $$ = $1;
		  }

  		| STRING attrlist
		  {
		    QyFilter filter;

		    filter = (QyFilter) XtNew(struct QyFilter);

		    filter->label = strdup($1);
		    filter->orfilt = $2;
		    
		    filter->andchain = (QyFilter) NULL;
		    $$ = filter;
		  }
		;

 andlist:	STRING attrlist '+' andlisttail
		  {
		    QyFilter filter;

		    filter = (QyFilter) XtNew(struct QyFilter);

		    filter->label = strdup($1);
		    filter->orfilt = $2;
		    
		    filter->andchain = $4;
		    $$ = filter;
		  }
		;

 andlisttail:	STRING attrlist '+' andlisttail
		  {
		    QyFilter filter;

		    filter = (QyFilter) XtNew(struct QyFilter);

		    filter->label = strdup($1);
		    filter->orfilt = $2;
		    
		    filter->andchain = $4;
		    $$ = filter;
		  }

		| STRING attrlist
		  {
		    QyFilter filter;
		    filter = (QyFilter) XtNew(struct QyFilter);

		    filter->label = strdup($1);
		    filter->orfilt = $2;
		    
		    filter->andchain = (QyFilter) NULL;
		    $$ = filter;
		  }
		;

 attrlist:	string_list
		  {
		    attrTypeList list = NULL;
		    stringCell str;
		    AttributeType type;

		    for (str = $1; str != NULLStrCell; str = str->next)
		      {
			type = str2AttrT(str->string);
			if (type != NULLAttrT)
			  QyATListAdd(type, &list);
			else
			  yyerror((char *) NULL);
		      }

		    $$ = list;
		    SeqFree(&$1);
		  }
		;

/**************************************************
 *
 * List by attribute value
 * 
 */

 list_attr_option: string_list ':' STRING ':' listval_list ':' string_list
			  {
			    BOOLEAN showrdn;
			    showrdn = (lexequ($3, "1") == 0? TRUE : FALSE);

			    AddListSelection($1, showrdn, $5, $7);
			  }
			| string_list ':' STRING ':' listval_list ':' 
			  {
			    BOOLEAN showrdn;
			    showrdn = (lexequ($3, "1") == 0? TRUE : FALSE);

			    AddListSelection($1, showrdn, $5, NULLStrCell);
			  }

			;

 listval_list:		listval
			  {
			    $$ = $1;
			  }
			|  listval_list ',' listval
			  {
			    $3->next = $1;
			    $$ = $3;
			  }
			| {$$ = (XLUlistValue) NULL;}
			;

 listval:		'+' STRING
   			  {
			    XLUlistValue listval;
			    
			    listval = XtNew(struct XLUlistValue);
			    listval->next = (XLUlistValue) NULL;
			    listval->type = ListAll;

			    listval->attrs = NULLStrCell;
			    SeqAddString($2, &listval->attrs);

			    $$ = listval;
			  }

			| STRING
			  {
			    XLUlistValue listval;
			    
			    listval = XtNew(struct XLUlistValue);
			    listval->next = (XLUlistValue) NULL;
			    listval->type = ListOne;

			    listval->attrs = NULLStrCell;
			    SeqAddString($1, &listval->attrs);

			    $$ = listval;
			  }

			| '(' string_list ')'
			  {
			    XLUlistValue listval;
			    
			    listval = XtNew(struct XLUlistValue);
			    listval->next = (XLUlistValue) NULL;
			    listval->type = ListOne;
			    listval->attrs = $2;

			    $$ = listval;
			  }

			| '+' '(' string_list ')'
			  {
			    XLUlistValue listval;
			    
			    listval = XtNew(struct XLUlistValue);
			    listval->next = (XLUlistValue) NULL;
			    listval->type = ListAll;
			    listval->attrs = $3;

			    $$ = listval;
			  }
			;

/***************************************************
 *
 * Read types
 *
 */

 read_option:		READ_TYPE ':' read_attr_list {;}
			;

 read_attr_list:	read_attr
			| read_attr_list ',' read_attr
			;

 read_attr:		STRING
			  {
			    add_display_attr($1, ShowAll);
			  }
			| '-' STRING
			  {
			    add_display_attr($2, ShowLongest);
			  }
			;

/**************************************************
 *
 * User defined attribute names
 * 
 */

 attr_name_option:	ATTR_NAME ':' attr_name_list {;}
			;
	
 attr_name_list:	attr_name
  			| attr_name_list ',' attr_name
  			;

 attr_name:		STRING STRING
			  {
			    add_attribute_name($1, $2);
			  }
			;

/**************************************************
 *
 * Modify configuration options
 * 
 */

 modify_option:		MODIFY_TYPE ':' modify_type_args	{;}

			| ATTR_TEMPLATE ':' attr_template_args	{;}

			| ATTR_GROUP ':' attr_group_args	{;}
			;


 modify_type_args: STRING ':' STRING ':' STRING ':' string_list ':' string_list
		     {
		       set_entry_class($1, $3, $5, $7, $9);
		       SeqFree(&$7);
		     }
		   | STRING ':' STRING ':' STRING ':' string_list ':'
                     {
		       set_entry_class($1, $3, $5, $7, NULLStrCell);
		       SeqFree(&$7);
		     }
                   ;

 attr_group_args: STRING ':' STRING ':' string_list
		     {
		       set_entry_group($1, $3, $5);
		       SeqFree(&$5);
		     }
		;

 attr_template_args:	string_list ':' STRING ':' string_list
			  {
			    set_attr_template($3, $1, $5);
			  }
			;

/**************************************************
 *
 * Miscellaneous rules
 * 
 */

 string_list:	STRING
		  {
		    $$ = NULLStrCell;
		    (void) SeqAddString($1, &($$));
		  }

		| string_list ',' STRING 
		  {
		    $$ = $1;
		    (void) SeqAddString($3, &($$));
		  }
		;

%%

/*****************************************************************************
 * yylex()
 *
 * Lexer for parser!
 *
 ****************************************************************************/

int yylex()
{
  register int c;
  int count = 0;
  FILE *lexfile = configuration_file;
  register String lexptr;
  BOOLEAN space;

  extern int _filbuf(), fseek();
  
  lexeme[0] = '\0';
  
  while (isspace((char) (c = getc(lexfile))))
    {
      if (c == EOF)
	return 0;

      if (c == '\n')
	config_line_num++;
    }
  
  lexeme[count++] = c;
  
  switch (c)
    {
    case '#':

      while ((c = getc(lexfile)) != '\n' && c != EOF)
	;

      config_line_num++;
      
      return yylex();

    case ':':
    case '(':
    case ')':
    case ',':
    case '{':
    case '}':
    case '-':
    case '+':

      lexeme[count] = '\0';
      return c;

    case '\"':

      count = 0;
      space = FALSE;

      while ((c = getc(lexfile)) != EOF && c != '\"')
	{
	  if (space == FALSE)
	    {
	      lexeme[count++] = (char) c;
	      
	      if (isspace((char) c))
		space = TRUE;
	    }
	  else if (!isspace((char) c))
	    {
	      lexeme[count++] = (char) c;
	      space = FALSE;
	    }
	}
    
      if (c == '\"')
	{
	  lexeme[count] = '\0'; 

	  (void) strcpy(yylval.strval, lexeme);
	  return STRING;
	} else
	  return 0;
    }

  while ((c = getc(lexfile)) != EOF && 
	 ! (c == '('
	    || c == ')'
	    || c == ':'
	    || c == '{'
	    || c == '}'
	    || c == ','
	    || c == '\"'
	    || isspace((char) c)))
    lexeme[count++] = (char) c;
  
  if (c == EOF)
    return 0;
  
  (void) fseek(lexfile, (long) -1, 1);
  
  lexeme[count] = '\0';
  
  if ((lexeme[0] == 'a' || lexeme[0] == 'A') && lexeme[1] == '.')
    {
      lexptr = (String) (lexeme + 2);

      switch (*lexptr)
	{
	case 'a':
	case 'A':

	  if (!lexequ("Activate", lexptr))
	    {
	      yylval.symbol = A_ACTIVATE;
	      return A_ACTIVATE;
	    }

	  if (!lexequ("AbortRequest", lexptr))
	    {
	      yylval.symbol = A_ABORT;
	      return A_ABORT;
	    }
      
	  break;
	  
	case 'c':
	case 'C':

	  if (!lexequ("CloseObject", lexptr))
	    {
	      yylval.symbol = A_CLOSEOBJ;
	      return A_CLOSEOBJ;
	    }
	  
	  break;
	  
	case 'k':
	case 'K':

	  if (!lexequ("KeepObject", lexptr))
	    {
	      yylval.symbol = A_KEEPOBJ;
	      return A_KEEPOBJ;
	    }
	  
	  break;

	case 'p':
	case 'P':

	  if (!lexequ("Pipeout", lexptr))
	    {
	      yylval.symbol = A_PIPEOUT;
	      return A_PIPEOUT;
	    }

	  break;

	case 'q':
	case 'Q':

	  if (!lexequ("QuitApp", lexptr))
	    {
	      yylval.symbol = A_QUITAPP;
	      return A_QUITAPP;
	    }
	  
	  break;
	  
	case 'r':
	case 'R':

	  if (!lexequ("RaiseObject", lexptr))
	    {
	      yylval.symbol = A_RAISEOBJ;
	      return A_RAISEOBJ;
	    }
	  
	  break;

	case 's':
	case 'S':

	  if (!lexequ("Select", lexptr))
	    {
	      yylval.symbol = A_SELECT;
	      return A_SELECT;
	    }
	  
	  break;

	default:

	  (void) strcpy(yylval.strval, lexeme);
	  return STRING;
	}

      (void) strcpy(yylval.strval, lexeme);
      return STRING;
    }
  else if ((lexeme[0] == 'l' || lexeme[0] == 'L') && lexeme[1] == '.')
    {
      lexptr = (String) (lexeme + 2);
      
      switch (*lexptr)
	{
	case 'a':
	case 'A':
	  
	  if (!lexequ("Add", lexptr))
	    {
	      yylval.symbol = ADD;
	      return ADD;
	    }
	  
	  break;

	case 'b':
	case 'B':
	  
	  if (!lexequ("Button", lexptr))
	    {
	      yylval.symbol = BUTTON;
	      return BUTTON;
	    }

	  if (!lexequ("Browse", lexptr))
	    {
	      yylval.symbol = BROWSE;
	      return BROWSE;
	    }
	  
	  break;

	case 'e':
	case 'E':
	  
	  if (!lexequ("EntryList", lexptr))
	    {
	      yylval.symbol = ENTRY_LIST;
	      return ENTRY_LIST;
	    }

	  if (!lexequ("ErrorView", lexptr))
	    {
	      yylval.symbol = ERROR_VIEW;
	      return ERROR_VIEW;
	    }
	  
	  break;

	case 'h':
	case 'H':
	  
	  if (!lexequ("HelpWindow", lexptr))
	    {
	      yylval.symbol = HELP;
	      return HELP;
	    }
	  
	  break;

	case 'l':
	case 'L':
	  
	  if (!lexequ("lookup", lexptr))
	    {
	      yylval.symbol = LOOKUP;
	      return LOOKUP;
	    }
	  
	  break;

	case 'm':
	case 'M':
	  
	  if (!lexequ("ModifyEntry", lexptr))
	    {
	      yylval.symbol = MODIFY;
	      return MODIFY;
	    }
	  
	  if (!lexequ("MovetoEntry", lexptr))
	    {
	      yylval.symbol = MOVETO;
	      return MOVETO;
	    }
	  
	  break;

	case 'r':
	case 'R':
	  
	  if (!lexequ("ReadDisplay", lexptr))
	    {
	      yylval.symbol = ENTRY_DISPLAY;
	      return ENTRY_DISPLAY;
	    }
	  
	  break;

	case 's':
	case 'S':
	  
	  if (!lexequ("StatusBar", lexptr))
	    {
	      yylval.symbol = STATUS;
	      return STATUS;
	    }
	  
	  if (!lexequ("SearchLog", lexptr))
	    {
	      yylval.symbol = SEARCH_LOG;
	      return SEARCH_LOG;
	    }
	  
	  break;

	case 't':
	case 'T':
	  
	  if (!lexequ("TitleBar", lexptr))
	    {
	      yylval.symbol = TITLE;
	      return TITLE;
	    }
	  
	  break;

	default:
	  
	  (void) strcpy(yylval.strval, lexeme);
	  return STRING;
	}

      (void) strcpy(yylval.strval, lexeme);
      return STRING;

    }
  else
    {
      lexptr = (String) lexeme;

      switch (*lexptr)
	{
	case 'A':
	case 'a':

	  if (!lexequ("action", lexptr))
	    {
	      yylval.symbol = ACTION;
	      return ACTION;
	    }

	  if (!lexequ("attrGroup", lexptr))
	    {
	      yylval.symbol = ATTR_GROUP;
	      return ATTR_GROUP;
	    }

	  if (!lexequ("attrValTemplate", lexptr))
	    {
	      yylval.symbol = ATTR_TEMPLATE;
	      return ATTR_TEMPLATE;
	    }

	  if (!lexequ("attrName", lexptr))
	    {
	      yylval.symbol = ATTR_NAME;
	      return ATTR_NAME;
	    }

	  if (!lexequ("APPdefFile", lexptr))
	    {
	      yylval.symbol = APPDEF_FILE;
	      return APPDEF_FILE;
	    }

	  break;
	  
	case 'b':
	case 'B':

	  if (!lexequ("bitmapDirectory", lexptr))
	    {
	      yylval.symbol = BITMAP_DIR;
	      return BITMAP_DIR;
	    }

	  if (!lexequ("bitmap", lexptr))
	    {
	      yylval.symbol = BITMAP;
	      return BITMAP;
	    }
	  
	  break;

	case 'c':
	case 'C':

	  if (!lexequ("current", lexptr))
	    {
	      yylval.symbol = CURRENT;
	      return CURRENT;
	    }
	
	  break;

	case 'd':
	case 'D':

	  if (!lexequ("DisplayAttribute", lexptr))
	    {
	      yylval.symbol = READ_TYPE;
	      return READ_TYPE;
	    }

	  if (!lexequ("DefaultTarget", lexptr))
	    {
	      yylval.symbol = DEFAULT_TARGET;
	      return DEFAULT_TARGET;
	    }

	  break;
	  
	case 'e':
	case 'E':

	  if (!lexequ("EntryDB", lexptr))
	    {
	      yylval.symbol = DNDB;
	      return DNDB;
	    }

	  if (!lexequ("expand", lexptr))
	    {
	      yylval.symbol = EXPAND;
	      return EXPAND;
	    }
	  
	  if (!lexequ("entryClass", lexptr))
	    {
	      yylval.symbol = MODIFY_TYPE;
	      return MODIFY_TYPE;
	    }

	  break;

	case 'G':
	case 'g':

	  if (!lexequ("globalmanaged", lexptr))
	    {
	      yylval.symbol = GLOBALMANAGED;
	      return GLOBALMANAGED;
	    }
	  
	  break;
	  
       	case 'h':
	case 'H':

	  if (!lexequ("helpDirectory", lexptr))
	    {
	      yylval.symbol = HELP_DIR;
	      return HELP_DIR;
	    }
	  
	  if (!lexequ("helpFile", lexptr))
	    {
	      yylval.symbol = HELPFILE;
	      return HELPFILE;
	    }

	  if (!lexequ("horizontal", lexptr))
	    {
	      yylval.symbol = HORIZONTAL;
	      return HORIZONTAL;
	    }
	  
	  if (!lexequ("height", lexptr))
	    {
	      yylval.symbol = HEIGHT;
	      return HEIGHT;
	    }
	  
	  break;
	  
	case 'i':
	case 'I':

	  if (!lexequ("insensitive", lexptr))
	    {
	      yylval.symbol = INSENSITIVE;
	      return INSENSITIVE;
	    }

	  break;


	case 'l':
	case 'L':

	  if (!lexequ("layout", lexptr))
	    {
	      yylval.symbol = LAYOUT;
	      return LAYOUT;
	    }
	  
	  if (!lexequ("label", lexptr))
	    {
	      yylval.symbol = LABEL;
	      return LABEL;
	    }
	  
	  if (!lexequ("localmanaged", lexptr))
	    {
	      yylval.symbol = LOCALMANAGED;
	      return LOCALMANAGED;
	    }
	  
	  if (!lexequ("ListAttribute", lexptr))
	    {
	      yylval.symbol = LIST_ATTR;
	      return LIST_ATTR;
	    }
	  
	  break;
	  
	case 'n':
	case 'N':

	  if (!lexequ("new", lexptr))
	    {
	      yylval.symbol = NEW;
	      return NEW;
	    }

	  break;

	case 'S':
	case 's':

	  if (!lexequ("searchClass", lexptr))
	    {
	      yylval.symbol = UFS_CLASS;
	      return UFS_CLASS;
	    }

	  if (!lexequ("searchRule", lexptr))
	    {
	      yylval.symbol = UFS_DEFAULTS;
	      return UFS_DEFAULTS;
	    }
	  
	  if (!lexequ("setname", lexptr))
	    {
	      yylval.symbol = NAME;
	      return NAME;
	    }
	  
	  if (!lexequ("start", lexptr))
	    {
	      yylval.symbol = START;
	      return START;
	    }
	  
	  break;

	case 't':
	case 'T':
	  
	  if (!lexequ("targetClass", lexptr))
	    {
	      yylval.symbol = UFS_TARGET;
	      return UFS_TARGET;
	    }

	  break;

	case 'u':
	case 'U':
	  
	  if (!lexequ("ufnPath", lexptr))
	    {
	      yylval.symbol = UFN_PATH;
	      return UFN_PATH;
	    }

	  break;

	case 'v':
	case 'V':

	  if (!lexequ("vertical", lexptr))
	    {
	      yylval.symbol = VERTICAL;
	      return VERTICAL;
	    }
	  
	  break;
	  
	case 'w':
	case 'W':

	  if (!lexequ("width", lexptr))
	    {
	      yylval.symbol = WIDTH;
	      return WIDTH;
	    }
	  
	  break;

	default:
	  (void) strcpy(yylval.strval, lexeme);
	  return STRING;
	}

      (void) strcpy(yylval.strval, lexeme);
      return STRING;
    }

}

/* ARGSUSED */
int yyerror(err)
     char *err;
{
  char msg[LINESIZE];

  (void) sprintf(msg,
 "Fatal parse error in \"%s\", line %d, at or near `%s'.\nError was: %s",
		 config_file_path,
		 config_line_num,
		 lexeme,
		 err);

  XLUpopupMessage(msg);

  exit(1);
  return 1;
} /* yylex */


