#ifndef _Xlu_configure_h_
#define _Xlu_configure_h_

#include "defs.h"
#include "windows.h"
#include "layout.h"
#include "actions.h"
#include "xdutil.h"

#include "attrt_list.h"
#include "sequence.h"

/***************************************************
 *
 * Used in configuration of window templates.
 *
 */

typedef enum 
{
  OT_action,
  OT_width,
  OT_height,
  OT_bitmap,
  OT_helpfile,
  OT_label,
  OT_name,
  OT_expand,
  OT_insensitive
} XLUpropertyType;

typedef struct XLUobjectProperty
{
  XLUpropertyType type;
  union
    {
      String action_name;
      Dimension width;
      Dimension height;
      String bitmap_file;
      String help_file;
      String label;
      String name;
      BOOLEAN expand;
      BOOLEAN insensitive;
    } resource;

  struct XLUobjectProperty *next;
} *XLUobjectProperty;

#define NULLObjRes (XLUobjectProperty) NULL
#define resource_alloc() XtNew(struct XLUobjectProperty)

/***************************************************
 *
 * Structures for configuring directory operations.
 *
 */

/***************************************************
 *
 * A group of attributes displayed in a modify pane.
 *
 */

typedef struct XLUattributeGroup
{
  String name;
  String description;

  attrTypeList attributes;
  struct XLUattributeGroup *next;
} *XLUattributeGroup;

#define NULLAttrGroup (XLUattributeGroup) NULL
#define attr_group_alloc() XtNew(struct XLUattributeGroup)

/***************************************************
 *
 * This groups objectClasses into a generic
 * entry type.
 *
 */

typedef struct XLUentryClass
{
  String name;

  AttributeType rdn_type;
  stringCell groups;

  attrTypeList oc_list;
  struct XLUentryClass *next;
} *XLUentryClass;

#define NULLEntryClass (XLUentryClass) NULL
#define entry_class_alloc() XtNew(struct XLUentryClass)

/***************************************************
 *
 * Attribute value template.
 *
 */

typedef struct XLUattrTemplate
{
  AttributeType type;
  
  stringCell class_names;
  stringCell defaults;

  struct XLUattrTemplate *next;
} *XLUattrTemplate;

#define NULLAttrTemplate (XLUattrTemplate) NULL
#define attr_tmpl_alloc() XtNew(struct XLUattrTemplate)

/***************************************************
 *
 *  Attributes that are to be displayed.
 *
 */

typedef enum {ShowLongest, ShowAll} XLUshowType;
typedef struct XLUreadAttribute
{
  AttributeType type;
  XLUshowType showtype;
  
  struct XLUreadAttribute *next;
} *XLUreadAttribute;

#define NULLReadAttribute (XLUreadAttribute) NULL
#define read_attr_alloc() XtNew(struct XLUreadAttribute)

/***************************************************
 *
 * Friendly names for attributes.
 *
 */

typedef struct XLUattributeName
{
  AttributeType type;
  String name;
  
  struct XLUattributeName *next;
} *XLUattributeName;

#define NULLAttributeName (XLUattributeName) NULL
#define attr_name_alloc() XtNew(struct XLUattributeName)

typedef enum {ListAll, ListOne} XLUlistType;
typedef struct XLUlistValue
{
  stringCell attrs;
  XLUlistType type;
  struct XLUlistValue *next;
} *XLUlistValue;

typedef struct XLUlistSelection
{
  stringCell oclasses;
  BOOLEAN showdn;
  XLUlistValue listvals;
  struct XLUlistSelection *next;
} *XLUlistSelection;

#define NULLListSelection (XLUlistSelection) NULL
#define list_sel_alloc() XtNew(struct XLUlistSelection)

/***************************************************
 *
 * Global functions..
 *
 */

void user_tailor();
void parse_win_config();

int add_template_entry();
void put_layout();

XLUlayoutObject add_prim_to_layout();
XLUlayoutObject add_branch_to_layout();

void add_to_startup_list();
void add_action();

XLUobjectProperty put_resource();
XLUactionList new_actlist();

void set_entry_group();
void set_entry_class();

XLUentryClass get_class_of_name();
void set_base_class();

void set_attr_oc_rule();
void set_attr_template();

void add_display_attr();
void add_attribute_name();

void add_ufs_search_class();
void AddClassSelection();
stringCell GetFullSelection();
XLUlistSelection GetListSelection();

void add_dn_db();
String get_dn_db();

#endif /* _Xlu_configure_h_ */

