#ifndef lint
static char *rcsid = "$Header: /a/izar/vol/fileservers/izar/izar4/src/local/X500/xd/work.motif/src/RCS/objects.c,v 1.1 94/02/01 16:37:59 ccsrdsm Exp Locker: ccsrdsm $";
#endif

/* $Header: /a/izar/vol/fileservers/izar/izar4/src/local/X500/xd/work.motif/src/RCS/objects.c,v 1.1 94/02/01 16:37:59 ccsrdsm Exp Locker: ccsrdsm $ */

/*****************************************************************************

  objects.c - Routines to manage and build windows.

*****************************************************************************/

#include "defs.h"
#include "windows.h"

#include <X11/cursorfont.h>
#include <X11/Xutil.h>
#include <X11/Shell.h>

#include <Xm/ScrolledW.h>
#include <Xm/PushB.h>
#include <Xm/Text.h>
#include <Xm/Label.h>
#include <Xm/Form.h>
#include <Xm/MenuShell.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>

#include "layout.h"
#include "objects.h"
#include "requests.h"

#include "showentry.h"
#include "showlist.h"

#include "status.h"
#include "errview.h"
#include "update1.h"
#include "xdutil.h"

#include "query.h"
#include "dn_list.h"

/***************************************************
 *
 * Globals.
 *
 */

int start_template_ids[MAXSTARTS];
int startup_number = 0;

XLUwindowTemplate win_templates[MAXTEMPLATES];
int template_number = 0;

/***************************************************
 *
 * Externals.
 *
 */

extern Widget toplevel;
extern XLUrequestRecord XLUrequestList;

void exit();

/**************************************************
 *
 * Linked list used to store XLUwindowObject
 * structures.
 * 
 */

typedef struct WinObjectList
{
  struct XLUwindowObject winobj;
  struct WinObjectList *prev;
  struct WinObjectList *next;
} *WinObjectList;

static WinObjectList XLUobjects = (WinObjectList) NULL;
static NextWinObjID = 0;

/***************************************************
 *
 * Local functions.
 *
 */

static void unparent_object_children();
static void delete_win_object();

/*****************************************************************************
 * XLUsetWindowEntry()
 *
 * Set the current entry of the ID\'ed XLU window object.
 *
 ****************************************************************************/

void XLUsetWindowEntry(objid, new_dn)
     int objid;
     entryList new_dn;
{
  XLUwindowObject winobj;
  entryList dn_copy = NULLEntryList, dn_next;

  winobj = XLUgetWindowObject(objid);
  
  if (new_dn == NULLEntryList)
    {
      winobj->current_entry = NULLEntryList;
      return;
    }

  /* Copy the first element of the DN list only */
  dn_next = new_dn->next;
  new_dn->next = NULLEntryList;

  dn_list_copy(new_dn, &dn_copy);
  new_dn->next = dn_next;

  if (winobj->current_entry != NULLEntryList)
    dn_list_free(&winobj->current_entry);

  winobj->current_entry = dn_copy;
} /* XLUsetWindowEntry */

/*****************************************************************************
 * XLUsetWindowErrors()
 *
 * Set the current error list of the ID\'ed XLU window object.
 *
 ****************************************************************************/

void XLUsetWindowErrors(objid, errors)
     int objid;
     QyErrorList errors;
{
  XLUwindowObject winobj;
  QyErrorList new_errors = QyCopyErrorList(errors);

  winobj = XLUgetWindowObject(objid);

  if (winobj->current_errors != NULLError)
    QyFreeErrorList(&(winobj->current_errors));

  winobj->current_errors = new_errors;
} /* XLUsetWindowErrors */

/*****************************************************************************
 * XLUsetWindowList()
 *
 * Set the current entry list of the ID\'ed XLU window object.
 *
 ****************************************************************************/

void XLUsetWindowList(objid, newlist)
     int objid;
     entryList newlist;
{
  XLUwindowObject winobj;
  entryList new_entry_list;

  winobj = XLUgetWindowObject(objid);
  dn_list_copy(newlist, &new_entry_list);

  if (winobj->current_list != NULLEntryList)
    dn_list_free(&winobj->current_list);

  winobj->current_list = new_entry_list;
} /* XLUsetWindowList */

/*****************************************************************************
 * XLUsetWindowSaveList()
 *
 * Set the current "saved" error list of the ID\'ed XLU window object.
 *
 ****************************************************************************/

void XLUsetWindowSaveList(objid, newlist)
     int objid;
     entryList newlist;
{
  XLUwindowObject winobj = XLUgetWindowObject(objid);

  if (winobj->current_save_list != NULLEntryList)
    dn_list_free(&winobj->current_save_list);

  dn_list_copy(newlist, &winobj->current_save_list); 
} /* XLUsetWindowSaveList */

/*****************************************************************************
 * XLUisWaiting()
 *
 * Return TRUE if the ID\'ed window object owns an outstanding
 * query engine request.
 *
 ****************************************************************************/

BOOLEAN XLUisWaiting(objid)
     int objid;
{
  XLUrequestRecord rec;

  for (rec = XLUrequestList; rec != NULLXLUrequestRecord; rec = rec->next)
    if (rec->objid == objid)
      return TRUE;

  return FALSE;
} /* XLUisWaiting */

/*****************************************************************************
 * XLUwait()
 *
 * Process X events while the given window object is waits for
 * a query engine request to complete.
 *
 ****************************************************************************/

void XLUwait(objid)
     int objid;
{
  XEvent event;
  extern XtAppContext app_context;

  while (XLUisWaiting(objid) == TRUE)
    {
      XtAppNextEvent(app_context, &event);
      XtDispatchEvent(&event);
    }
} /* XLUwait */

/*****************************************************************************
 * XLUnewWindowObject()
 *
 * Create a new XLUwindowObject record, with new ID.
 *
 ****************************************************************************/

XLUwindowObject XLUnewWindowObject(id_ptr)
     int *id_ptr;
{
  WinObjectList newlist;

  /* New window object ID. */
  *id_ptr = NextWinObjID++;

  /**************************************************
   *
   * Create new list structure and add to head
   * of chain.
   * 
   */
  
  newlist = (WinObjectList) XtCalloc(1, sizeof(struct WinObjectList));
  newlist->winobj.id = *id_ptr;
  
  newlist->prev = (WinObjectList) NULL;
  newlist->next = XLUobjects;

  if (XLUobjects != (WinObjectList) NULL)
    XLUobjects->prev = newlist;

  XLUobjects = newlist;
  return &newlist->winobj;
} /* XLUnewWindowObject */

/*****************************************************************************
 * XLUgetWindowObject()
 *
 * Find the XLUwindowObject structure of the ID\'ed object.
 *
 ****************************************************************************/

XLUwindowObject XLUgetWindowObject(id)
     int id;
{
  WinObjectList obj;

  for (obj = XLUobjects; obj != (WinObjectList) NULL; obj = obj->next)
    if (obj->winobj.id == id)
      return &obj->winobj;

  return (XLUwindowObject) NULL;
} /* XLUgetWindowObject */

/*****************************************************************************
 * XLUcloseWindowObject()
 *
 * The ID\'ed window object has been deleted. Remove it\'s record.
 *
 ****************************************************************************/

void XLUcloseWindowObject(id)
     int id;
{
  WinObjectList del;

  for (del = XLUobjects; del != (WinObjectList) NULL; del = del->next)
    if (del->winobj.id == id)
      break;

  delete_win_object(&del->winobj);

  if (del->prev != (WinObjectList) NULL)
    {
      del->prev->next = del->next;
    }
  else
    XLUobjects = del->next;

  if (del->next != (WinObjectList) NULL)
    {
      del->next->prev = del->prev;
    }

  XtFree(del);
} /* XLUcloseWindowObject */

/*****************************************************************************
 * XLUmakeWindowObject()
 *
 * Create an XLU window using the identified template.
 *
 ****************************************************************************/

Widget XLUmakeWindowObject(tmpl_id, parentwin, parent_id_ptr, child_id_ptr)
     int tmpl_id;
     Widget parentwin;
     int *parent_id_ptr;
     int *child_id_ptr;
{
  int objid;
  XLUwindowObject winobj, parent;
  childList siblings;

  winobj = XLUnewWindowObject(&objid);

  /**************************************************
   *
   * Create and fill in XLUwindowObject structure.
   * 
   */

  (void) XLUmakeWindowFromTemplate(tmpl_id, objid, parentwin);

  winobj->current_entry = NULLEntryList;
  winobj->current_errors = NULLError;
  winobj->current_list = winobj->current_save_list = NULLEntryList;
  winobj->template_id = tmpl_id;

  if (parent_id_ptr != (int *) NULL)
    {
      winobj->parent_id = *parent_id_ptr;
      winobj->is_child = TRUE;
      
      parent = XLUgetWindowObject(*parent_id_ptr);
      siblings = parent->children;
      
      parent->children = child_list_alloc();
      parent->children->child_object_id = objid;
      parent->children->left_siblings = siblings;
    }
  else
    {
      winobj->parent_id = 0;
      winobj->is_child = FALSE;
    }

  winobj->children = NULLChildList;
  winobj->is_kept = FALSE;

  *child_id_ptr = objid;
  return winobj->parent_shell;
} /* XLUmakeWindowObject */

/*****************************************************************************
 * CreateStartupWindows()
 *
 * Create startup windows as configured in xlurc. The first window
 * in the list uses the "toplevel" shell.
 *
 * This function should only ever be called once!
 *
 ****************************************************************************/

void CreateStartupWindows()
{
  int count, objid;
  Widget curr_shell;
  static BOOLEAN used_toplevel = FALSE;
  XLUwindowObject currobj;

  extern char username[];
  extern Widget toplevel;

  if (startup_number == 0)
    xluFatal("No start up windows specified in configure files!");

  for (count = 0; count < startup_number; count++)
    {
      if (used_toplevel == TRUE)
        curr_shell = NULLWidget;
      else
        {
          curr_shell = toplevel;
          used_toplevel = TRUE;
        }

      (void) XLUmakeWindowObject(start_template_ids[count],
				 curr_shell,
				 (int *) NULL,
				 &objid);

      currobj = XLUgetWindowObject(count);

      /* Set the initial entry to be the user\'s entry */
      currobj->current_entry = NULLEntryList;

      displayCurrentList(count);
      updateErrorViewer(count);

      curr_shell = currobj->parent_shell;
      XtPopup(curr_shell, XtGrabNone);

      displayEntryTemplate(NULLWidget, count, 0, 0, NULLActionList);

      doPrimitiveSensitivity(count);
      XLUwait(count);

      displayCurrentEntry(NULLWidget, count, 0, 0, NULLActionList);
    }
} /* CreateStartupWindows */

/*****************************************************************************
 * free_child_list()
 *
 * 
 *
 ****************************************************************************/

void free_child_list(child_list_ptr)
     childList child_list_ptr;
{
  if (child_list_ptr == NULLChildList)
    return;

  free_child_list(child_list_ptr->left_siblings);
  XtFree((char *)child_list_ptr);
} /* free_child_list */

/*****************************************************************************
 * unparent_children()
 *
 * 
 *
 ****************************************************************************/

static void unparent_object_children(parent_obj_id)
     int parent_obj_id;
{
  childList children;
  XLUwindowObject currobj;

  currobj = XLUgetWindowObject(parent_obj_id);
  children = currobj->children;

  while (children != (childList) NULL)
    {
      currobj = XLUgetWindowObject(children->child_object_id);
      currobj->parent_id = MAXWINDOWS + 1;
      currobj->is_child = FALSE;
      children = children->left_siblings;
    }
} /* unparent_object_children */

/*****************************************************************************
 * free_widget_tree()
 *
 ****************************************************************************/

void free_widget_tree(widget_tree_ptr)
     WidgetTree widget_tree_ptr;
{
  if (widget_tree_ptr == NULLWidgetTree)
    return;

  free_widget_tree(widget_tree_ptr->descendants);
  free_widget_tree(widget_tree_ptr->left_siblings);

  XtFree((char *) widget_tree_ptr);
} /* free_widget_tree */

/*****************************************************************************
 * delete_win_object()
 *
 * Delete the contents of an XLUwindowObject structure and destroy
 * the associated window.
 *
 ****************************************************************************/

static void delete_win_object(objptr)
     XLUwindowObject objptr;
{
  XLUwindowObject parentobj;
  childList child_of_parent, delete;

  if (objptr == NULLWinObject)
    return;

  unparent_object_children(objptr->id);
 
  if (objptr->is_child)
    {
      parentobj = XLUgetWindowObject(objptr->parent_id);
      child_of_parent = parentobj->children;
      
      if (child_of_parent != NULLChildList)
	if (child_of_parent->child_object_id == objptr->id)
	  {
	    parentobj->children =  child_of_parent->left_siblings;
	    XtFree((char *) child_of_parent);
	  }

      while (child_of_parent != NULLChildList && 
	     child_of_parent->left_siblings != NULLChildList)
	{
	  if (child_of_parent->left_siblings->child_object_id == objptr->id)
	    {
	      delete = child_of_parent->left_siblings;
	      child_of_parent->left_siblings = delete->left_siblings;
	      
	      XtFree((char *) delete);
	      break;
	    }
	  child_of_parent = child_of_parent->left_siblings;
	}
    }

  free_widget_tree(objptr->widget_layout);
  free_child_list(objptr->children);
  
  /* Free current DN, DN lists and error list. */
  if (objptr->current_entry != NULLEntryList)
    dn_list_free(&objptr->current_entry);

  if (objptr->current_list != NULLEntryList) 
    dn_list_free(&objptr->current_list);

  if (objptr->current_save_list != NULLEntryList) 
    dn_list_free(&objptr->current_save_list);

  /* Close and destroy the window */
  if (objptr->parent_shell != NULLWidget)
    {
      if (XtIsRealized(objptr->parent_shell))
	{
	  XtUnmapWidget(objptr->parent_shell);
	  
	  if (objptr->parent_shell != toplevel)
	    XtDestroyWidget(objptr->parent_shell);
	}
    }
} /* delete_win_object */

/*****************************************************************************
 * XLUfindUnkeptChild()
 *
 * 
 *
 ****************************************************************************/

int XLUfindUnkeptChild(parent_obj_id, child_template_id)
     int parent_obj_id;
     int child_template_id;
{
  XLUwindowObject winobj;
  childList children;

  winobj = XLUgetWindowObject(parent_obj_id);
  children  = winobj->children;

  while (children != (childList) NULL)
    {
      winobj = XLUgetWindowObject(children->child_object_id);

      if (winobj->template_id == child_template_id && 
	  winobj->is_kept == FALSE)
	return children->child_object_id;
      
      children = children->left_siblings;
    }

  /* Failed ! */
  return (MAXWINDOWS + 1);
} /* XLUfindUnkeptChild */

/*****************************************************************************
 * XLUfindUnkeptObject()
 *
 * 
 *
 ****************************************************************************/

int XLUfindUnkeptObject(child_template_id)
     int child_template_id;
{
  WinObjectList listobj;
  XLUwindowObject currobj;

  for (listobj = XLUobjects;
       listobj != (WinObjectList) NULL;
       listobj = listobj->next)
    {
      currobj = &listobj->winobj;
      if (currobj->template_id == child_template_id &&
	  currobj->is_kept == FALSE)
	return currobj->id;
    }

  /* Failed ! */
  return (MAXWINDOWS + 1);
} /* XLUfindUnkeptObject */

/*****************************************************************************
 * XLUgetWindowObjectId()
 *
 * 
 *
 ****************************************************************************/

int XLUgetWindowObjectId(w)
     Widget w;
{
  Widget shell;
  WinObjectList listobj;
  XLUwindowObject currobj;

  if (w == NULLWidget)
    return 0;
  
  for (shell = w;
       shell != NULLWidget &&
       XtClass(shell) != topLevelShellWidgetClass &&
       XtClass(shell) != applicationShellWidgetClass &&
       shell != toplevel;
       shell = XtParent(shell))
    ;

  for (listobj = XLUobjects;
       listobj != (WinObjectList) NULL;
       listobj = listobj->next)
    {
      currobj = &listobj->winobj;
      if (currobj->parent_shell == shell)
	return currobj->id;
    }

  return MAXWINDOWS+1;
} /* XLUgetWindowObjectId */

