#ifndef lint
static char *rcsid = "$Header: /a/izar/vol/fileservers/izar/izar4/src/local/X500/xd/work.motif/src/RCS/status.c,v 1.1 94/02/01 16:38:19 ccsrdsm Exp Locker: ccsrdsm $";
#endif

/* $Header: /a/izar/vol/fileservers/izar/izar4/src/local/X500/xd/work.motif/src/RCS/status.c,v 1.1 94/02/01 16:38:19 ccsrdsm Exp Locker: ccsrdsm $ */

/*****************************************************************************

  status.c - Display status messages

*****************************************************************************/

#include "defs.h"

#include <X11/Shell.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/ScrolledW.h>
#include <Xm/Text.h>
#include <Xm/MessageB.h>

#include "query.h"

#include "windows.h"
#include "objects.h"
#include "callbacks.h"
#include "configure.h"
#include "actions.h"
#include "y.tab.h"
#include "layout.h"
#include "xdutil.h"
#include "xutil.h"
#include "ask.h"
#include "searchlog.h"

#include "status.h"
#include "showentry.h"

/**************************************************
 *
 * Local functions.
 * 
 */

static void okCB();

/**************************************************
 *
 * Local data.
 * 
 */

static BOOLEAN msgdone;

/*****************************************************************************
 * XLUstatusMessage()
 *
 * Display a message in the status bar of the identified window object.
 * If the window doesn\'t possess a status bar display the message
 * in a popup window.
 *
 ****************************************************************************/

void XLUstatusMessage(object_id, message)
     int object_id;
     String message;
{
  WidgetList statuswidgets;
  Widget currstatus;
  int aindex;
  Arg args[MAXARGS];
  XmString labelstr;
  String start, end;
  char save;
  XLUwindowObject winobj;

  /**************************************************
   *
   * If the given object_id doesn\'t refer to
   * an existing window, display the message
   * as a popup.
   *
   */

  winobj = XLUgetWindowObject(object_id);
  if (winobj == (XLUwindowObject) NULL)
    {
      XLUpopupMessage(message);
      return;
    }

  /**************************************************
   *
   * Try to locate a status bar primitive in the
   * given window object.
   * 
   */
     
  statuswidgets = XLUgetPrimitiveList(object_id, STATUS);

  /* If no status bar found, display a popup message. */
  if (statuswidgets == (Widget *) NULL)
    {
      XLUpopupMessage(message);
      return;
    }

  /**************************************************
   *
   * Create a compound string from a string
   * that may contain newlines.
   *
   */

  labelstr = XmStringCreateSimple("");

  start = message;
  while (!isnull(*start))
    {
      /* Get to new line or end of string */
      for (end = start; !isnull(*end) && *end != '\n'; end++)
	;
      
      /* Append string */
      save = *end;
      *end = '\0';

      XLUappendXmString(&labelstr, start);
      *end = save;

      if (*end == '\n')
	{
	  /* Append separator */
	  XLUappendSeparatorXmString(&labelstr);
	  start = end + 1;
	}
      else
	start = end;
    }

  /**************************************************
   *
   * Set message string.
   * 
   */

  for (aindex = 0, currstatus = statuswidgets[0];
       currstatus != NULLWidget;
       aindex++, currstatus = statuswidgets[aindex])
    {
      XtSetArg(args[0], XmNlabelString, labelstr);
      XtSetValues(currstatus, args, 1);
    }

  /**************************************************
   *
   * All done!
   * 
   */

  XmStringFree(labelstr);
  XtFree((char *) statuswidgets);
} /* XLUstatusMessage */

/*****************************************************************************
 * XLUpopupMessage()
 *
 * Display a message in a modal window.
 *
 ****************************************************************************/

void XLUpopupMessage(message)
     String message;
{
  Arg args[MAXARGS];
  XmString labelstr;
  Widget msg, shell, btn;
  XEvent event;
  String start, end;
  char save;
  XSizeHints szhints;
  long hintflags;

  extern Widget toplevel;
  extern XtAppContext app_context;
  extern Display *dpy;

  msgdone = FALSE;
  shell = XtCreatePopupShell("XLU message",
			     transientShellWidgetClass,
			     toplevel,
			     args,
			     0);

  /**************************************************
   *
   * Create a compound string from a string
   * that may contain newlines.
   *
   */

  labelstr = XmStringCreateSimple("");

  start = message;
  while (!isnull(*start))
    {
      /* Get to new line or end of string */
      for (end = start; !isnull(*end) && *end != '\n'; end++)
	;
      
      /* Append string */
      save = *end;
      *end = '\0';

      XLUappendXmString(&labelstr, start);
      *end = save;

      if (save == '\n')
	{
	  /* Append separator */
	  XLUappendSeparatorXmString(&labelstr);
	  start = end + 1;
	}
      else
	start = end;
    }

  /**************************************************
   *
   * Create message box.
   * 
   */

  labelstr = XmStringCreateSimple(message);
  XtSetArg(args[0], XmNmessageString, labelstr);
  XtSetArg(args[1], XmNdialogType, XmDIALOG_INFORMATION);
  msg = XmCreateMessageBox(shell, "XLU message", args, 2);

  XtAddCallback(msg, XmNokCallback, okCB, (XtPointer) NULL);
  XmStringFree(labelstr);

  /* Disable help and cancel buttons */
  btn = XmMessageBoxGetChild(msg, XmDIALOG_CANCEL_BUTTON);
  XtSetSensitive(btn, FALSE);

  btn = XmMessageBoxGetChild(msg, XmDIALOG_HELP_BUTTON);
  XtSetSensitive(btn, FALSE);

  /**************************************************
   *
   * Set WM options.
   * 
   */

  /* Raise message box */
  XtManageChild(msg);
  XtPopup(shell, XtGrabExclusive);

  /* Set user defined start position (can be ignored by WM). */
  XGetWMNormalHints(dpy, XtWindow(shell), &szhints, &hintflags);
  szhints.flags = USPosition | PSize | PWinGravity;
  XSetWMNormalHints(dpy, XtWindow(shell), &szhints);

  /**************************************************
   *
   * Process events until message box has
   * been lowered.
   * 
   */

  while (msgdone == FALSE)
    {
      XtAppNextEvent(app_context, &event);
      XtDispatchEvent(&event);
    }

  XtPopdown(shell);
  XtDestroyWidget(shell);
} /* XLUpopupMessage */

/*****************************************************************************
 * okCB()
 *
 * Signals demise of message dialog.
 *
 ****************************************************************************/

/* ARGSUSED */
static void okCB(w, closure, calldata)
     Widget w;
     XtPointer closure;
     XtPointer calldata;
{
  msgdone = TRUE;
} /* okCB */
