#ifndef lint
static char *rcsid = "$Header: /a/izar/vol/fileservers/izar/izar4/src/local/X500/xd/work.motif/src/RCS/xutil.c,v 1.1 94/02/01 16:38:28 ccsrdsm Exp Locker: ccsrdsm $";
#endif

/* $Header: /a/izar/vol/fileservers/izar/izar4/src/local/X500/xd/work.motif/src/RCS/xutil.c,v 1.1 94/02/01 16:38:28 ccsrdsm Exp Locker: ccsrdsm $ */

/*******************************************************************

  xutil.c - Miscellaneous Xt related functions.

*******************************************************************/

#include "defs.h"
#include <ctype.h>

#include "windows.h"
#include "xdutil.h"
#include "xutil.h"
#include "objects.h"

#include "quipu/common.h"
#include <Xm/Form.h>

/***************************************************
 *
 * Externals.
 *
 */

extern Widget toplevel;
extern Display *dpy;

extern long strtol();

/***************************************************
 *
 * Local data.
 *
 */

static char err_msg[LINESIZE];

/***************************************************
 *
 * Local functions.
 *
 */

static void bitmap_format_warning();
static void deleteContextEntry();
static void XLUfreePrimitiveData();

/*****************************************************************************
 * get_widget_geometry()
 *
 * Get the dimensions and parent releative position of a widget.
 *
 ****************************************************************************/

void get_widget_geometry(widget, width_ptr, height_ptr, x_ptr, y_ptr)
     Widget widget;
     Dimension *width_ptr;
     Dimension *height_ptr;
     Position *x_ptr;
     Position *y_ptr;
{
  Arg args[MAXARGS];
  int count;
  
  if (widget == NULLWidget)
    {
      if (width_ptr) *width_ptr = (Dimension) 0;
      if (height_ptr) *height_ptr = (Dimension) 0;

      if (x_ptr) *x_ptr = (Position) 0;
      if (y_ptr) *y_ptr = (Position) 0;

      return;
    }

  count = 0;
  if (width_ptr)
    {
      XtSetArg(args[count], XmNwidth, width_ptr); count++;
    }

  if (height_ptr)
    {
      XtSetArg(args[count], XmNheight, height_ptr); count++;
    }

  if (x_ptr)
    {
      XtSetArg(args[count], XmNx, x_ptr); count++;
    }

  if (y_ptr)
    {
      XtSetArg(args[count], XmNy, y_ptr); count++;
    }

  XtGetValues(widget, args, count);
} /* get_widget_geometry */

/*****************************************************************************
 * set_widget_dimensions()
 *
 * Force the dimensions of a widget (disregarding
 * management by a parent widget).
 *
 ****************************************************************************/

void set_widget_dimensions(widget, width, height)
     Widget widget;
     Dimension width;
     Dimension height;
{
  Arg args[MAXARGS];
  int count;
  Dimension old_width, old_height, border_width;

  if (widget == NULLWidget) return;

  count = 0;
  if (width == 0)
    {
      XtSetArg(args[count], XmNwidth, &old_width); count++;
    }

  if (height == 0)
    {
      XtSetArg(args[count], XmNheight, &old_height); count++;
    }

  XtSetArg(args[count], XmNborderWidth, &border_width); count++;
  XtGetValues(widget, args, count);

  if (width == 0) width = old_width;
  if (height == 0) height = old_height;

  count = 0;

  XtSetArg(args[count], XmNwidth, width); count++;
  XtSetArg(args[count], XmNheight, height); count++;

  XtSetValues(widget, args, count);
  XtResizeWidget(widget, width, height, border_width);
} /* set_widget_dimensions */

/*****************************************************************************
 * create_background_from_bitmap_file()
 *
 * Load a file bitmap into the given widget.
 *
 ****************************************************************************/

void create_background_from_bitmap_file(widget, bitmap_file_name)
     Widget widget;
     String bitmap_file_name;
{
  Arg args[MAXARGS];
  Dimension width, height;
  Pixmap background;
  XmString cstr;

  background = create_bitmap_from_file(bitmap_file_name,
				       &width,
				       &height,
				       widget);

  if (background == XmUNSPECIFIED_PIXMAP)
    return;

  XtSetArg(args[0], XmNwidth, width);
  XtSetArg(args[1], XmNheight, height);

  XtSetArg(args[2], XmNbackgroundPixmap, background);
  
  cstr = XmStringCreateSimple("");
  XtSetArg(args[3], XmNlabelString, cstr);

  XtSetValues(widget, args, 4);
  XmStringFree(cstr);

  XtSetArg(args[0], XmNresizable, FALSE);
  XtSetValues(widget, args, 1);

  XtAddCallback(widget,
		XmNdestroyCallback,
		XLUfreePixmap,
		(XtPointer) background);
} /* create_background_from_bitmap_file */

/*****************************************************************************
 * create_bitmap_from_file()
 *
 * Create a Pixmap from the given file.
 *
 ****************************************************************************/

Pixmap create_bitmap_from_file(bitmap_file_name,
			       width_ptr,
			       height_ptr,
			       widget)
     String bitmap_file_name;
     Dimension *width_ptr;
     Dimension *height_ptr;
     Widget widget;
{
  FILE *bitmap_file;
  Dimension width, height;
  int bytes_wide, byte_count, count;
  char line[STRINGLEN];
  register String start, end;
  unsigned char *bytes, *byte_ptr;
  long temp_byte;
  Screen *screen;
  Pixmap return_pixmap;

  bitmap_file = fopen(bitmap_file_name, "r");
  if (bitmap_file == NULL)
    {
      (void) sprintf(err_msg, "Can't open bitmap file `%s'", bitmap_file_name);
      xluWarning(err_msg);

      (void) fclose(bitmap_file);
      return XmUNSPECIFIED_PIXMAP;
    }

  if (fgets(line, STRINGLEN, bitmap_file) != NULL)
    {
      end = line;

      while (*end != '\0' && *end != '\n')
	end++;

      *end = '\0';
      start = end - 1;
      
      while (isdigit(*start))
	start--;

      start++;
      width = (int) atoi(start);
    }
  else
    {
      bitmap_format_warning(bitmap_file_name);
      (void) fclose(bitmap_file);
      return XmUNSPECIFIED_PIXMAP;
    }

  if (fgets(line, STRINGLEN, bitmap_file) != NULL)
    {
      end = line;

      while (*end != '\0' && *end != '\n')
	end++;

      *end = '\0';
      start = end - 1;

      while (isdigit(*start))
	start--;

      start++;
      height = (int) atoi(start);
    }
  else
    {
      bitmap_format_warning(bitmap_file_name);
      (void) fclose(bitmap_file);
      return XmUNSPECIFIED_PIXMAP;
    }
  
  bytes_wide = width / 8;

  if ((width % 8) > 0)
    bytes_wide++;

  byte_count = bytes_wide * height;
  byte_ptr = bytes = (unsigned char *) XtMalloc((unsigned) byte_count);
  
  while (1)
    {
      if (fgets(line, STRINGLEN, bitmap_file) == NULL)
	{
	  bitmap_format_warning(bitmap_file_name);
	  (void) fclose(bitmap_file);
	  
	  XtFree((char *) bytes);
	  return XmUNSPECIFIED_PIXMAP;
	}
      
      if (index(line, '{') != NULL) break;
    }

  count = 0;
  while (count < byte_count)
    {
      if (fgets(line, STRINGLEN, bitmap_file) == NULL)
	{
	  bitmap_format_warning(bitmap_file_name);
	  (void) fclose(bitmap_file);

	  XtFree((char *) bytes);
	  return  XmUNSPECIFIED_PIXMAP;
	}
    
      start = end = line;
      while (*end != '\0' && *end != '\n' && *end != ';')
	{
	  if ((start = index(start, 'x')) == NULL)
	    {
	      bitmap_format_warning(bitmap_file_name);
	      (void) fclose(bitmap_file);

	      XtFree((char *) bytes);
	      return XmUNSPECIFIED_PIXMAP;
	    }
	  
	  start++;
	  end = start + 2;
	  
	  temp_byte = strtol(start, (char **) NULL, 16);

	  if (temp_byte < 0 || temp_byte > 255)
	    {
	      bitmap_format_warning(bitmap_file_name);
	      (void) fclose(bitmap_file);

	      XtFree((char *) bytes);
	      return XmUNSPECIFIED_PIXMAP;
	    }

	  *byte_ptr++ = temp_byte;
	  count++;
	  end++;
	}
    }
  
  *width_ptr = width;
  *height_ptr = height;
  
  (void) fclose(bitmap_file);

  screen = XtScreen(toplevel);
  return_pixmap = XCreatePixmapFromBitmapData(dpy,
					      XtWindow(widget),
					      (char *) bytes,
					      (int) width,
					      (int) height,
					      BlackPixelOfScreen(screen),
					      WhitePixelOfScreen(screen),
					      DefaultDepthOfScreen(screen));

  XtFree((char *) bytes);
  return return_pixmap;
}

/*****************************************************************************
 * get_dims_of_bitmap_file()
 *
 * Get the dimensions of the bitmap contained in the given file.
 *
 ****************************************************************************/

BOOLEAN get_dims_of_bitmap_file(bitmap_file_name, width_ptr, height_ptr)
     String bitmap_file_name;
     int *width_ptr;
     int *height_ptr;
{
  FILE *bitmap_file;
  Dimension width, height;
  char line[STRINGLEN];
  register String start, end;

  bitmap_file = fopen(bitmap_file_name, "r");
  if (bitmap_file == NULL)
    {
      (void) sprintf(err_msg,
		     "Cannot open bitmap file '%s'.\n", 
		     bitmap_file_name);

      xluWarning(err_msg);
      return FALSE;
    }

  if (fgets(line, STRINGLEN, bitmap_file) != NULL)
    {
      end = line;
      while (*end != '\0' && *end != '\n') end++;
      *end = '\0';
      
      start = end - 1;
      
      while (isdigit(*start)) start--;
      start++;

      width = (int) atoi(start);
    }
  else
    {
      bitmap_format_warning(bitmap_file_name);
      return FALSE;
    }

  if (fgets(line, STRINGLEN, bitmap_file) != NULL)
    {
      end = line;
      while (*end != '\0' && *end != '\n') end++;
      *end = '\0';
      
      start = end - 1;
      
      while (isdigit(*start)) start--;
      start++;
      
      height = (int) atoi(start);
    }
  else
    {
      return FALSE;
    }
  
  *width_ptr = (int) width;
  *height_ptr = (int) height;

  return TRUE;
}

static void bitmap_format_warning(filename)
     String filename;
{
  (void) sprintf(err_msg,
		 "Incorrect format in bitmap file `%s'.",
		 filename);
      
  xluWarning(err_msg);
}

/*****************************************************************************
 * XLUgetTextExtents()
 *
 * Get dimensions of the given text string when using the font
 * in the gicen widget.
 *
 ****************************************************************************/

void XLUgetTextExtents(w, text, width_ptr, height_ptr)
     Widget w;
     String text;
     Dimension *width_ptr;
     Dimension *height_ptr;
{
  XmFontList font;
  XmString cstring;
  Arg args[MAXARGS];
  register String start, end;
  char save;
  int linecount;

  XtSetArg(args[0], XmNfontList, &font);
  XtGetValues(w, args, 1);

  if (font == (XmFontList) NULL)
    {
      *width_ptr = *height_ptr = 1;
      return;
    }

  cstring = XmStringCreateSimple("");

  start = text;
  while (!isnull(*start))
    {
      for (end = start + 1; !isnull(*end) && *end != '\n'; end++)
	;

      save = *end;
      *end = '\0';

      XLUappendXmString(&cstring, start);
      *end = save;

      if (save == '\n')
	XLUappendSeparatorXmString(&cstring);
      
      if (save == '\n')
	start = ++end;
      else
	start = end;
    }
  
  *width_ptr = XmStringWidth(font, cstring);
  *height_ptr = XmStringHeight(font, cstring);

  linecount = 0;
  start = text;

  while (!isnull(*start))
    {
      for (end = start + 1; !isnull(*end) && *end != '\n'; end++)
	;

      linecount++;
      start = end;
    }

  *height_ptr *= linecount;
  XmStringFree(cstring);
} /* XLUgetTextExtents */

/*****************************************************************************
 * XLUappendXmString()
 *
 * Concatenate text to a compund string.
 *
 ****************************************************************************/

void XLUappendXmString(cstr, text)
     XmString *cstr;
     String text;
{
  XmString newstr, oldstr, appstr;

  oldstr = *cstr;
  if (oldstr == (XmString) NULL)
    return;

  appstr = XmStringCreateSimple(text);
  newstr = XmStringConcat(oldstr, appstr);

  XmStringFree(oldstr);
  XmStringFree(appstr);
  
  *cstr = newstr;
} /* XLUappendXmString */

/*****************************************************************************
 * XLUappendSeparatorXmString()
 *
 * Append a separator to the given compound string.
 *
 ****************************************************************************/

void XLUappendSeparatorXmString(cstr)
     XmString *cstr;
{
  XmString appstr, newstr, oldstr;

  oldstr = *cstr;
  if (oldstr == (XmString) NULL)
    return;

  appstr = XmStringSeparatorCreate();
  newstr = XmStringConcat(oldstr, appstr);

  XmStringFree(oldstr);
  XmStringFree(appstr);
  
  *cstr = newstr;
} /* XLUappendSeparatorXmString */

/*****************************************************************************
 * XLUfreeStorage()
 *
 * Default callback used to free dynamically allocated memory.
 * Usually attached to the XmNdestroyCallback list.
 *
 ****************************************************************************/

/* ARGSUSED */
void XLUfreeStorage(w, closure, calldata)
     Widget w;
     XtPointer closure;
     XtPointer calldata;
{
  XtFree((char *) closure);
} /* XLUfreeStorage */

/*****************************************************************************
 * XLUaddContextEntry()
 *
 * Add an XContext to the given widget, and add a callback
 * that removes the conext when the widget is destroyed.
 *
 ****************************************************************************/

void XLUaddContextEntry(w, cid, data, destroy)
     Widget w;
     XContext cid;
     caddr_t data;
     XtCallbackProc destroy;
{
  caddr_t tmp_data;

  if (w == NULLWidget)
    return;

  if (XFindContext(dpy, XtWindow(w), cid, &tmp_data) == XCNOENT)
    {
      XtAddCallback(w, XmNdestroyCallback, deleteContextEntry, (XtPointer)cid);
    }
  else
    {
      /* Assumes that the same destroy function is used. */
      if (destroy != (XtCallbackProc) NULL)
	{
	  XtRemoveCallback(w,
			   XmNdestroyCallback,
			   destroy,
			   (XtPointer) tmp_data);
	}
    }

  if (destroy != (XtCallbackProc) NULL)
    XtAddCallback(w, XmNdestroyCallback, destroy, (XtPointer) data);

  XSaveContext(dpy, XtWindow(w), cid, data);
} /* XLUaddContextEntry */

/*****************************************************************************
 * deleteContextEntry()
 *
 * Delete an XContext from the given widget. Used by XLUaddContextEntry
 * to add to an XmNdestroyCallback list.
 *
 ****************************************************************************/

/* ARGSUSED */
static void deleteContextEntry(w, closure, calldata)
     Widget w;
     XtPointer closure;
     XtPointer calldata;
{
  XDeleteContext(dpy, XtWindow(w), (XContext) closure);
} /* deleteContextEntry */

/*****************************************************************************
 * XLUremoveContextEntry()
 *
 * Remove a context entry.
 *
 ****************************************************************************/

void XLUremoveContextEntry(w, context)
     Widget w;
     XContext context;
{
  XDeleteContext(dpy, XtWindow(w), context);
} /* XLUremoveContextEntry */

/*****************************************************************************
 * XLUfreePixmap()
 *
 * XtCallback frees a pixmap.
 *
 ****************************************************************************/

/* ARGSUSED */
void XLUfreePixmap(w, closure, calldata)
{
  XFreePixmap(dpy, (Pixmap) closure);
} /* XLUfreePixmap */

/*****************************************************************************
 * XLUgetListString()
 *
 * Retrieve a selected string (assumes that multiple selections aren't
 * permitted).
 *
 ****************************************************************************/

String XLUgetListString(listbox, stridx)
     Widget listbox;
     int *stridx;
{
  Arg args[MAXARGS];
  int count, itemcount;
  XmStringTable selected, items;
  int objid;
  String str;

  /**************************************************
   *
   * Get selected compound string.
   * 
   */

  XtSetArg(args[0], XmNselectedItems, &selected);
  XtGetValues(listbox, args, 1);

  if (selected == (XmStringTable) NULL)
    return NULLString;

  objid = XLUgetWindowObjectId(listbox);

  /**************************************************
   *
   * Get text of selected compound string.
   * 
   */

  str = XLUgetXmStringText(selected[0]);

  /**************************************************
   *
   * Get index of selected string.
   * 
   */

  XtSetArg(args[0], XmNitems, &items);
  XtSetArg(args[1], XmNitemCount, &itemcount);
  XtGetValues(listbox, args, 2);

  for (count = 0; count < itemcount; count++)
    if (XmStringCompare(selected[0], items[count]) == TRUE)
      break;

  *stridx = count;
  return str;
} /* XLUgetListString */

/*****************************************************************************
 * XLUgetXmStringText()
 *
 * Return the text of a compound string.
 *
 ****************************************************************************/

String XLUgetXmStringText(cstring)
     XmString cstring;
{
  XmStringContext strctx;
  String str = NULLString;
  XmStringCharSet chset;
  XmStringDirection direction;
  Boolean separator;

  (void) XmStringInitContext(&strctx, cstring);
  XmStringGetNextSegment(strctx, &str, &chset, &direction, &separator);

  XtFree((char *) direction);
  XtFree((char *) separator);
  XtFree((char *) chset);

  XmStringFreeContext(strctx);
  return str;
} /* XLUgetXmStringText */

/*****************************************************************************
 * XLUpostMenu()
 *
 * Post a popup menu at the bottom left hand corner of the widget w;
 *
 ****************************************************************************/

void XLUpostMenu(w, popup, event)
     Widget w;
     Widget popup;
     XButtonEvent event;
{
  Dimension height;
  Position x, y;
  Arg args[MAXARGS];
  int xint, yint;
  Window win;

  XtSetArg(args[0], XmNheight, &height);
  XtSetArg(args[1], XmNx, &x);
  XtSetArg(args[2], XmNy, &y);
  XtGetValues(w, args, 3);

  yint = y;
  xint = x;

  yint += height;

  XTranslateCoordinates(dpy,
			XtWindow(XtParent(w)),
			XtScreen(w)->root,
			xint, yint,
			&xint, &yint,
			&win);
  
  event.x_root = xint;
  event.y_root = yint;

  XmMenuPosition(popup, event);
  XtManageChild(popup);
} /* XLUpostMenu */

/*****************************************************************************
 * XLUplaceWindow()
 *
 * Compute staggered window placement for a new XLU window of
 * given width and height.
 *
 ****************************************************************************/

void XLUplaceWindow(width, height, x_ptr, y_ptr)
     int width;
     int height;
     int *x_ptr;
     int *y_ptr;
{
  Screen *scr;
  int scrwidth, scrheight;

#define XBASE 48
#define YBASE 48
#define XINC  24
#define YINC  24

  static int xoff = XBASE, yoff = YBASE;

  /* Get screen dimensions. */
  scr = DefaultScreenOfDisplay(dpy);
  scrheight = HeightOfScreen(scr);
  scrwidth = WidthOfScreen(scr);

  /* Increment X and Y offsets until windows goes off edge of screen */
  xoff += XINC;
  yoff += YINC;

  if ((xoff + width) > scrwidth || (yoff + height) > scrheight)
    {
      xoff = XBASE;
      yoff = YBASE;
    }

  *x_ptr = xoff;
  *y_ptr = yoff;
} /* XLUplaceWindow */

/*****************************************************************************
 * XLUsetWMHandler()
 *
 * Set event handler for WM events in top level windows.
 *
 ****************************************************************************/

void XLUsetWMHandler(widget)
     Widget widget;
{
  extern Atom wm_delete_window;

  XtOverrideTranslations(widget,
	     XtParseTranslationTable("<Message>WM_PROTOCOLS: XLUwmEvent()"));

  (void) XSetWMProtocols(dpy, XtWindow(widget), &wm_delete_window, 1);
} /* XLUsetWMHandler */

/*****************************************************************************
 * XLUsavePrimitiveContext()
 *
 * Save context information associated with a primitive window object.
 *
 ****************************************************************************/

void XLUsavePrimitiveContext(w, type, actid)
     Widget w;
     int type;
     int actid;
{
  Arg args[MAXARGS];
  XLUprimitiveData pdata;
  XtPointer ptr;

  XtSetArg(args[0], XmNuserData, &ptr);
  XtGetValues(w, args, 1);

  if (ptr == (XtPointer) NULL)
    /* First time save */
    {
      /* Allocate new data structure and save it */
      pdata = (XLUprimitiveData) XtCalloc(1, sizeof(struct XLUprimitiveData));
      
      XtSetArg(args[0], XmNuserData, (XtPointer) pdata);
      XtSetValues(w, args, 1);

      /* ...and add a callback to clean up user data */
      XtAddCallback(w,
		    XmNdestroyCallback,
		    XLUfreePrimitiveData,
		    (XtPointer) pdata);
    }
  else
    /* Overwriting old data */
    {
      pdata = (XLUprimitiveData) ptr;
    }

  pdata->actid = actid;
  pdata->type = type;
} /* XLUsavePrimitiveContext */

/*****************************************************************************
 * XLUsavePrimitiveData()
 *
 * Save "userdata" along with the standard information. The "cleanup"
 * callback is used to free this.
 * 
 ****************************************************************************/

void XLUsavePrimitiveData(w, userdata, cleanup)
     Widget w;
     XtPointer userdata;
     XtCallbackProc cleanup;
{
  Arg args[MAXARGS];
  XLUprimitiveData pdata;
  XtPointer ptr;

  XtSetArg(args[0], XmNuserData, &ptr);
  XtGetValues(w, args, 1);

  pdata = (XLUprimitiveData) ptr;

  if (pdata->userdata != (XtPointer) NULL)
    (*cleanup)(w, pdata->userdata, (XtPointer) NULL);

  pdata->userdata = userdata;
  pdata->cleanup = cleanup;
} /* XLUsavePrimitiveData */

/*****************************************************************************
 * XLUgetPrimitiveType()
 *
 * Get the primitive type identifier.
 *
 ****************************************************************************/

int XLUgetPrimitiveType(w)
     Widget w;
{
  Arg args[MAXARGS];
  XtPointer ptr;
  XLUprimitiveData pdata;
  
  XtSetArg(args[0], XmNuserData, &ptr);
  XtGetValues(w, args, 1);

  if (ptr == (XtPointer) NULL)
    /* Shouldn\'t happen */
    return 0;

  pdata = (XLUprimitiveData) ptr;
  return pdata->type;
} /* XLUgetPrimitiveType */

/*****************************************************************************
 * XLUgetPrimitiveActionID()
 *
 * Get the action ID associated with the given primitive object.
 *
 ****************************************************************************/

int XLUgetPrimitiveActionID(w)
     Widget w;
{
  Arg args[MAXARGS];
  XtPointer ptr;
  XLUprimitiveData pdata;
  
  XtSetArg(args[0], XmNuserData, &ptr);
  XtGetValues(w, args, 1);

  if (ptr == (XtPointer) NULL)
    /* Shouldn\'t happen */
    return ActionUndefined;

  pdata = (XLUprimitiveData) ptr;
  return pdata->actid;
} /* XLUgetPrimitiveActionID */

/*****************************************************************************
 * XLUgetPrimitiveData()
 *
 * Get the user data associated with the given primitive object.
 *
 ****************************************************************************/

XtPointer XLUgetPrimitiveData(w)
     Widget w;
{
  Arg args[MAXARGS];
  XtPointer ptr;
  XLUprimitiveData pdata;
  
  XtSetArg(args[0], XmNuserData, &ptr);
  XtGetValues(w, args, 1);

  if (ptr == (XtPointer) NULL)
    /* Shouldn\'t happen */
    return (XtPointer) NULL;

  pdata = (XLUprimitiveData) ptr;
  return pdata->userdata;
} /* XLUgetPrimitiveData */

/*****************************************************************************
 * XLUfreePrimitiveData()
 *
 * Clean up callback for XLU primitive objects.
 *
 ****************************************************************************/

/* ARGSUSED */
static void XLUfreePrimitiveData(w, closure, calldata)
     Widget w;
     XtPointer closure;
     XtPointer calldata;
{
  XLUprimitiveData pdata;

  pdata = (XLUprimitiveData) closure;

  if (pdata->cleanup != (XtCallbackProc) NULL)
    /* Clean up user data */
    (*pdata->cleanup)(w, pdata->userdata, (XtPointer) NULL);

  XtFree((char *) pdata);
} /* XLUfreePrimitiveData */
